@@ -1,7 +1,7 @@
 // ==========================================================================
 //                 SeqAn - The Library for Sequence Analysis
 // ==========================================================================
-// Copyright (c) 2006-2016, Knut Reinert, FU Berlin
+// Copyright (c) 2006-2024, Knut Reinert, FU Berlin
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,7 @@
 #ifndef SEQAN_INCLUDE_SEQAN_ALIGN_DP_META_INFO_H_
 #define SEQAN_INCLUDE_SEQAN_ALIGN_DP_META_INFO_H_
 
-namespace seqan {
+namespace seqan2 {
 
 // ============================================================================
 // Forwards
@@ -444,10 +444,42 @@ template <typename TDPMetaColumn>
 struct TrackingEnabled_<TDPMetaColumn, LastCell>:
     IsTrackingEnabled_<typename TDPMetaColumn::TLastCell_>{};
 
+// ----------------------------------------------------------------------------
+// Metafunction LastColumnEnabled_
+// ----------------------------------------------------------------------------
+
+template <typename TAlgo, typename TColumnDescriptor>
+struct LastColumnEnabled_
+{
+    typedef typename IsSameType<typename TColumnDescriptor::TColumnProperty, DPFinalColumn>::Type Type;
+};
+
+template <typename TAlgo, typename TGapSpec, typename TTraceSpec, typename TColumnDescriptor>
+struct LastColumnEnabled_<DPProfile_<TAlgo, TGapSpec, TTraceSpec>, TColumnDescriptor> :
+    LastColumnEnabled_<TAlgo, TColumnDescriptor>
+{};
+
+// ----------------------------------------------------------------------------
+// Metafunction LastRowEnabled_
+// ----------------------------------------------------------------------------
+
+template <typename TAlgo, typename TCellDescriptor, typename TColumnDescriptor>
+struct LastRowEnabled_
+{
+    typedef typename And<IsSameType<TCellDescriptor, LastCell>,
+                         Or<IsSameType<typename TColumnDescriptor::TLocation, PartialColumnBottom>,
+                            IsSameType<typename TColumnDescriptor::TLocation, FullColumn> > >::Type Type;
+};
+
+template <typename TAlgo, typename TGapSpec, typename TTraceSpec, typename TCellDescriptor, typename TColumnDescriptor>
+struct LastRowEnabled_<DPProfile_<TAlgo, TGapSpec, TTraceSpec>, TCellDescriptor, TColumnDescriptor> :
+    LastRowEnabled_<TAlgo, TCellDescriptor, TColumnDescriptor>
+{};
+
 // ============================================================================
 // Functions
 // ============================================================================
 
-}  // namespace seqan
+}  // namespace seqan2
 
 #endif  // #ifndef SEQAN_INCLUDE_SEQAN_ALIGN_DP_META_INFO_H_
