@@ -35,7 +35,14 @@
 #ifndef INDEX_FM_RANK_DICTIONARY_BASE_H_
 #define INDEX_FM_RANK_DICTIONARY_BASE_H_
 
-namespace seqan {
+namespace seqan2 {
+
+// ============================================================================
+// Forwards
+// ============================================================================
+
+template <typename TText, typename TSpec, typename TConfig>
+struct LF;
 
 // ============================================================================
 // Tags
@@ -54,6 +61,12 @@ namespace seqan {
  *
  * @tag RankDictionaryFibres#FibreRanks
  * @brief The string set containing a bit string for each character.
+ *
+ * @tag RankDictionaryFibres#FibreSuperBlocks
+ * @brief The string set containing a bit string for each character.
+ *
+ * @tag RankDictionaryFibres#FibreUltraBlocks
+ * @brief The string set containing a bit string for each character.
  */
 
 struct FibreRanks_;
@@ -96,12 +109,12 @@ struct RDConfig
  * @class RankDictionary
  * @headerfile <seqan/index.h>
  * @brief A rank dictionary is a data structure storing the rank of an element in a sequence at every position of
- *        the sequence.
+ *        the sequence. Only supports sequences containing @link SimpleType @endlink or bool.
  *
  * @signature template <typename TValue, typename TSpec>
  *            struct RankDictionary;
  *
- * @tparam TSpec The rank dictionary specialisation. Default: @link WaveletTree @endlink, @link TwoLevelRankDictionary @endlink.
+ * @tparam TSpec The rank dictionary specialisation. Default: @link WaveletTree @endlink, @link Levels @endlink.
  */
 template <typename TValue, typename TSpec>
 struct RankDictionary;
@@ -249,12 +262,91 @@ createRankDictionary(RankDictionary<TValue, TSpec> & dict, TText const & text)
     TTextIterator textBegin = begin(text, Standard());
     TTextIterator textEnd = end(text, Standard());
     for (TTextIterator textIt = textBegin; textIt != textEnd; ++textIt)
-        setValue(dict, textIt - textBegin, value(textIt));
+        setValue(dict, textIt - textBegin, *textIt);
 
     // Update all ranks.
     updateRanks(dict);
 }
 
+template <typename TText, typename TSpec, typename TConfig, typename TOtherText, typename TSA>
+inline void
+createRankDictionary(LF<TText, TSpec, TConfig> & lf, TOtherText const & text, TSA const & sa)
+{
+    typedef typename GetValue<TSA>::Type                    TSAValue;
+    typedef typename Size<TSA>::Type                        TSize;
+
+    // Resize the RankDictionary.
+    resize(lf.bwt, lengthSum(text) + 1, Exact());
+
+    // Assign the text value by value.
+    setValue(lf.bwt, 0, back(text));
+
+    for (TSize i = 0; i < length(sa); ++i)
+    {
+        TSAValue pos = sa[i];
+
+        if (pos != 0)
+        {
+            setValue(lf.bwt, i + 1, getValue(text, pos - 1));
+        }
+        else
+        {
+            setValue(lf.bwt, i + 1, lf.sentinelSubstitute);
+            lf.sentinels = i + 1;
+        }
+    }
+
+   // Update all ranks.
+   updateRanks(lf.bwt);
+}
+
+template <typename TText, typename TSSetSpec, typename TSpec, typename TConfig, typename TOtherText, typename TSA>
+inline void
+createRankDictionary(LF<StringSet<TText, TSSetSpec>, TSpec, TConfig> & lf, TOtherText const & text, TSA const & sa)
+{
+    typedef typename Value<TSA>::Type                       TSAValue;
+    typedef typename Size<TSA>::Type                        TSize;
+
+    // Resize the RankDictionary.
+    TSize seqNum = countSequences(text);
+    TSize totalLen = lengthSum(text);
+    resize(lf.sentinels, seqNum + totalLen, Exact());
+    resize(lf.bwt, seqNum + totalLen, Exact());
+
+    // Fill the sentinel positions (they are all at the beginning of the bwt).
+    for (TSize i = 0; i < seqNum; ++i)
+    {
+        if (length(text[seqNum - (i + 1)]) > 0)
+        {
+            setValue(lf.bwt, i, back(text[seqNum - (i + 1)]));
+            setValue(lf.sentinels, i, false);
+        }
+    }
+
+    // Compute the rest of the bwt.
+    for (TSize i = 0; i < length(sa); ++i)
+    {
+        TSAValue pos;    // = SA[i];
+        posLocalize(pos, sa[i], stringSetLimits(text));
+
+        if (getSeqOffset(pos) != 0)
+        {
+            setValue(lf.bwt, i + seqNum, getValue(getValue(text, getSeqNo(pos)), getSeqOffset(pos) - 1));
+            setValue(lf.sentinels, i + seqNum, false);
+        }
+        else
+        {
+            setValue(lf.bwt, i + seqNum, lf.sentinelSubstitute);
+            setValue(lf.sentinels, i + seqNum, true);
+        }
+    }
+
+   // Update all ranks.
+   updateRanks(lf.bwt);
+   // Update the auxiliary RankDictionary of sentinel positions.
+   updateRanks(lf.sentinels);
+}
+
 // ----------------------------------------------------------------------------
 // Function getRank()
 // ----------------------------------------------------------------------------
