@@ -1,7 +1,7 @@
 // ==========================================================================
 //                 SeqAn - The Library for Sequence Analysis
 // ==========================================================================
-// Copyright (c) 2006-2016, Knut Reinert, FU Berlin
+// Copyright (c) 2006-2024, Knut Reinert, FU Berlin
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,12 +33,13 @@
 #ifndef SEQAN_HEADER_MISC_SET_H
 #define SEQAN_HEADER_MISC_SET_H
 
+#include <functional>
 #include <set>
 #include <seqan/misc/base.h>
 
 //////////////////////////////////////////////////////////////////////////////
 
-namespace seqan
+namespace seqan2
 {
 
     //////////////////////////////////////////////////////////////////////////////
@@ -370,7 +371,7 @@ namespace seqan
 
 
     template <typename TElement>
-    struct SetLess_ : public std::binary_function<TElement, TElement, bool>
+    struct SetLess_ : public std::function<bool(TElement, TElement)>
     {
         // key less
         inline bool operator() (TElement const &a, TElement const &b) {
